@@ -13,13 +13,11 @@ from gunicorn.config import Config, get_default_config_file
 from gunicorn import debug
 from gunicorn.six import execfile_
 
-
-class Application(object):
-    """\
+class BaseApplication(object):
+    """
     An application interface for configuring and loading
     the various necessities for any given web framework.
     """
-
     def __init__(self, usage=None, prog=None):
         self.usage = usage
         self.cfg = None
@@ -29,14 +27,56 @@ class Application(object):
         self.do_load_config()
 
     def do_load_config(self):
+        """
+        Loads the configuration
+        """
         try:
+            self.load_default_config()
             self.load_config()
         except Exception as e:
             sys.stderr.write("\nError: %s\n" % str(e))
             sys.stderr.flush()
             sys.exit(1)
 
-    def load_config_from_file(self, filename):
+    def load_default_config(self):
+        # init configuration
+        self.cfg = Config(self.usage, prog=self.prog)
+
+    def init(self, parser, opts, args):
+        raise NotImplementedError
+
+    def load(self):
+        raise NotImplementedError
+
+    def load_config(self):
+        """
+        This method is used to load the configuration from one or several input(s).
+        Custom Command line, configuration file.
+        You have to override this method in your class.
+        """
+        raise NotImplementedError
+
+    def reload(self):
+        self.do_load_config()
+        if self.cfg.spew:
+            debug.spew()
+
+    def wsgi(self):
+        if self.callable is None:
+            self.callable = self.load()
+        return self.callable
+
+    def run(self):
+        try:
+            Arbiter(self).run()
+        except RuntimeError as e:
+            sys.stderr.write("\nError: %s\n\n" % e)
+            sys.stderr.flush()
+            sys.exit(1)
+
+class Application(BaseApplication):
+
+    def get_config_from_filename(self, filename):
 
         if not os.path.exists(filename):
             raise RuntimeError("%r doesn't exist" % filename)
@@ -55,6 +95,22 @@ class Application(object):
             traceback.print_exc()
             sys.exit(1)
 
+        return cfg
+
+    def get_config_from_module_name(self, module_name):
+        return util.import_module(module_name).__dict__
+
+    def load_config_from_module_name_or_filename(self, location):
+        """
+        Loads the configuration file: the file is a python file, otherwise raise an RuntimeError
+        Exception or stop the process if the configuration file contains a syntax error.
+        """
+
+        try:
+            cfg = self.get_config_from_module_name(module_name=location)
+        except ImportError:
+            cfg = self.get_config_from_filename(filename=location)
+
         for k, v in cfg.items():
             # Ignore unknown names
             if k not in self.cfg.settings:
@@ -67,10 +123,12 @@ class Application(object):
 
         return cfg
 
-    def load_config(self):
-        # init configuration
-        self.cfg = Config(self.usage, prog=self.prog)
+    def load_config_from_file(self, filename):
+        return self.load_config_from_module_name_or_filename(
+            location=filename
+        )
 
+    def load_config(self):
         # parse console args
         parser = self.cfg.parser()
         args = parser.parse_args()
@@ -99,22 +157,6 @@ class Application(object):
                 continue
             self.cfg.set(k.lower(), v)
 
-    def init(self, parser, opts, args):
-        raise NotImplementedError
-
-    def load(self):
-        raise NotImplementedError
-
-    def reload(self):
-        self.do_load_config()
-        if self.cfg.spew:
-            debug.spew()
-
-    def wsgi(self):
-        if self.callable is None:
-            self.callable = self.load()
-        return self.callable
-
     def run(self):
         if self.cfg.check_config:
             try:
@@ -140,9 +182,4 @@ class Application(object):
                 if pythonpath not in sys.path:
                     sys.path.insert(0, pythonpath)
 
-        try:
-            Arbiter(self).run()
-        except RuntimeError as e:
-            sys.stderr.write("\nError: %s\n\n" % e)
-            sys.stderr.flush()
-            sys.exit(1)
+        super(Application, self).run()
