@@ -12,9 +12,9 @@ except ImportError: # python 2.6
     from . import argparse_compat as argparse
 import os
 import pwd
+import ssl
 import sys
 import textwrap
-import types
 
 from gunicorn import __version__
 from gunicorn.errors import ConfigError
@@ -48,6 +48,7 @@ class Config(object):
         self.settings = make_settings()
         self.usage = usage
         self.prog = prog or os.path.basename(sys.argv[0])
+        self.env_orig = os.environ.copy()
 
     def __getattr__(self, name):
         if name not in self.settings:
@@ -76,24 +77,40 @@ class Config(object):
                 help="show program's version number and exit")
         parser.add_argument("args", nargs="*", help=argparse.SUPPRESS)
 
-        keys = list(self.settings)
-        def sorter(k):
-            return (self.settings[k].section, self.settings[k].order)
-
         keys = sorted(self.settings, key=self.settings.__getitem__)
         for k in keys:
             self.settings[k].add_option(parser)
 
         return parser
 
+    @property
+    def worker_class_str(self):
+        uri = self.settings['worker_class'].get()
+
+        ## are we using a threaded worker?
+        is_sync = uri.endswith('SyncWorker') or uri == 'sync'
+        if is_sync and self.threads > 1:
+            return "threads"
+        return uri
+
     @property
     def worker_class(self):
         uri = self.settings['worker_class'].get()
+
+        ## are we using a threaded worker?
+        is_sync = uri.endswith('SyncWorker') or uri == 'sync'
+        if is_sync and self.threads > 1:
+            uri = "gunicorn.workers.gthread.ThreadWorker"
+
         worker_class = util.load_class(uri)
         if hasattr(worker_class, "setup"):
             worker_class.setup()
         return worker_class
 
+    @property
+    def threads(self):
+        return self.settings['threads'].get()
+
     @property
     def workers(self):
         return self.settings['workers'].get()
@@ -122,8 +139,18 @@ class Config(object):
     @property
     def logger_class(self):
         uri = self.settings['logger_class'].get()
-        logger_class = util.load_class(uri, default="simple",
-            section="gunicorn.loggers")
+        if uri == "simple":
+            # support the default
+            uri = "gunicorn.glogging.Logger"
+        
+        # if statsd is on, automagically switch to the statsd logger
+        if 'statsd_host' in self.settings:
+            logger_class = util.load_class("gunicorn.instrument.statsd.Statsd",
+                section="gunicorn.loggers")
+        else:
+            logger_class = util.load_class(uri,
+                default="gunicorn.glogging.Logger",
+                section="gunicorn.loggers")
 
         if hasattr(logger_class, "install"):
             logger_class.install()
@@ -136,14 +163,30 @@ class Config(object):
     @property
     def ssl_options(self):
         opts = {}
-        if self.certfile:
-            opts['certfile'] = self.certfile
-
-        if self.keyfile:
-            opts['keyfile'] = self.keyfile
-
+        for name, value in self.settings.items():
+            if value.section == 'Ssl':
+                opts[name] = value.get()
         return opts
 
+    @property
+    def env(self):
+        raw_env = self.settings['raw_env'].get()
+        env = {}
+
+        if not raw_env:
+            return env
+
+        for e in raw_env:
+            s = six.bytes_to_str(e)
+            try:
+                k, v = s.split('=', 1)
+            except ValueError:
+                raise RuntimeError("environment setting %r invalid" % s)
+
+            env[k] = v
+
+        return env
+
 
 class SettingMeta(type):
     def __new__(cls, name, bases, attrs):
@@ -367,13 +410,56 @@ def validate_post_request(val):
     else:
         raise TypeError("Value must have an arity of: 4")
 
+
+def validate_chdir(val):
+    # valid if the value is a string
+    val = validate_string(val)
+
+    # transform relative paths
+    path = os.path.abspath(os.path.normpath(os.path.join(util.getcwd(), val)))
+
+    # test if the path exists
+    if not os.path.exists(path):
+        raise ConfigError("can't chdir to %r" % val)
+
+    return path
+
+
+def validate_file(val):
+    if val is None:
+        return None
+
+    # valid if the value is a string
+    val = validate_string(val)
+
+     # transform relative paths
+    path = os.path.abspath(os.path.normpath(os.path.join(util.getcwd(), val)))
+
+    # test if the path exists
+    if not os.path.exists(path):
+        raise ConfigError("%r not found" % val)
+
+    return path
+
+def validate_hostport(val):
+    val = validate_string(val)
+    if val is None:
+        return None
+    elements = val.split(":")
+    if len(elements) == 2:
+        return (elements[0], int(elements[1]))
+    else:
+        raise TypeError("Value must consist of: hostname:port")
+
 def get_default_config_file():
-    config_path = os.path.join(os.path.abspath(os.getcwd()), 'gunicorn.conf.py')
+    config_path = os.path.join(os.path.abspath(os.getcwd()),
+            'gunicorn.conf.py')
     if os.path.exists(config_path):
         return config_path
     return None
 
 
+# Please remember to run "make html" in docs/ after update "desc" attributes.
 class ConfigFile(Setting):
     name = "config"
     section = "Config File"
@@ -382,7 +468,7 @@ class ConfigFile(Setting):
     validator = validate_string
     default = None
     desc = """\
-        The path to a Gunicorn config file.
+        The path to a Gunicorn config file, or python module.
 
         Only has an effect when specified on the command line or as part of an
         application specific configuration.
@@ -443,13 +529,16 @@ class Workers(Setting):
     meta = "INT"
     validator = validate_pos_int
     type = int
-    default = 1
+    default = int(os.environ.get('WEB_CONCURRENCY', 1))
     desc = """\
         The number of worker process for handling requests.
 
         A positive integer generally in the 2-4 x $(NUM_CORES) range. You'll
         want to vary this a bit to find the best for your particular
         application's work load.
+
+        By default, the value of the WEB_CONCURRENCY environment variable. If
+        it is not defined, the default is 1.
         """
 
 
@@ -473,7 +562,7 @@ class WorkerClass(Setting):
 
         * ``sync``
         * ``eventlet`` - Requires eventlet >= 0.9.7
-        * ``gevent``   - Requires gevent >= 0.12.2 (?)
+        * ``gevent``   - Requires gevent >= 0.13
         * ``tornado``  - Requires tornado >= 0.2
 
         Optionally, you can provide your own worker by giving gunicorn a
@@ -483,6 +572,26 @@ class WorkerClass(Setting):
         can also load the gevent class with ``egg:gunicorn#gevent``
         """
 
+class WorkerThreads(Setting):
+    name = "threads"
+    section = "Worker Processes"
+    cli = ["--threads"]
+    meta = "INT"
+    validator = validate_pos_int
+    type = int
+    default = 1
+    desc = """\
+        The number of worker threads for handling requests.
+
+        Run each worker with the specified number of threads.
+
+        A positive integer generally in the 2-4 x $(NUM_CORES) range. You'll
+        want to vary this a bit to find the best for your particular
+        application's work load.
+
+        If it is not defined, the default is 1.
+        """
+
 
 class WorkerConnections(Setting):
     name = "worker_connections"
@@ -637,11 +746,30 @@ class Debug(Setting):
     desc = """\
         Turn on debugging in the server.
 
-        This limits the number of worker processes to 1 and changes some error
-        handling that's sent to clients.
+        **DEPRECATED**: This no functionality was removed after v18.0.
+        This option is now a no-op.
         """
 
 
+class Reload(Setting):
+    name = "reload"
+    section = 'Debugging'
+    cli = ['--reload']
+    validator = validate_bool
+    action = 'store_true'
+    default = False
+    desc = '''\
+        Restart workers when code changes.
+
+        This setting is intended for development. It will cause workers to be
+        restarted whenever application code changes.
+
+        The reloader is incompatible with application preloading. When using a
+        paste configuration be sure that the server block does not import any
+        application code or the reload will not work as designed.
+        '''
+
+
 class Spew(Setting):
     name = "spew"
     section = "Debugging"
@@ -685,6 +813,17 @@ class PreloadApp(Setting):
         """
 
 
+class Chdir(Setting):
+    name = "chdir"
+    section = "Server Mechanics"
+    cli = ["--chdir"]
+    validator = validate_chdir
+    default = util.getcwd()
+    desc = """\
+        Chdir to specified directory before apps loading.
+        """
+
+
 class Daemon(Setting):
     name = "daemon"
     section = "Server Mechanics"
@@ -699,6 +838,25 @@ class Daemon(Setting):
         background.
         """
 
+class Env(Setting):
+    name = "raw_env"
+    action = "append"
+    section = "Server Mechanics"
+    cli = ["-e", "--env"]
+    meta = "ENV"
+    validator = validate_list_string
+    default = []
+
+    desc = """\
+        Set environment variable (key=value).
+
+        Pass variables to the execution environment. Ex.::
+
+            $ gunicorn -b 127.0.0.1:8000 --env FOO=1 test:app
+
+        and test for the foo variable environment in your application.
+        """
+
 
 class Pidfile(Setting):
     name = "pidfile"
@@ -713,6 +871,18 @@ class Pidfile(Setting):
         If not set, no PID file will be written.
         """
 
+class WorkerTmpDir(Setting):
+    name = "worker_tmp_dir"
+    section = "Server Mechanics"
+    cli = ["--worker-tmp-dir"]
+    meta = "DIR"
+    validator = validate_string
+    default = None
+    desc = """\
+        A directory to use for the worker heartbeat temporary file.
+
+        If not set, the default temporary directory will be used.
+        """
 
 class User(Setting):
     name = "user"
@@ -808,26 +978,15 @@ class SecureSchemeHeader(Setting):
         """
 
 
-class XForwardedFor(Setting):
-    name = "x_forwarded_for_header"
-    section = "Server Mechanics"
-    meta = "STRING"
-    validator = validate_string
-    default = 'X-FORWARDED-FOR'
-    desc = """\
-        Set the X-Forwarded-For header that identify the originating IP
-        address of the client connection to gunicorn via a proxy.
-        """
-
-
 class ForwardedAllowIPS(Setting):
     name = "forwarded_allow_ips"
     section = "Server Mechanics"
+    cli = ["--forwarded-allow-ips"]
     meta = "STRING"
     validator = validate_string_to_list
     default = "127.0.0.1"
     desc = """\
-        Front-end's IPs from which allowed to handle X-Forwarded-* headers.
+        Front-end's IPs from which allowed to handle set secure headers.
         (comma separate).
 
         Set to "*" to disable checking of Front-end IPs (useful for setups
@@ -856,29 +1015,29 @@ class AccessLogFormat(Setting):
     cli = ["--access-logformat"]
     meta = "STRING"
     validator = validate_string
-    default = '"%(h)s %(l)s %(u)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
+    default = '%(h)s %(l)s %(u)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
     desc = """\
-        The Access log format .
+        The access log format.
 
-        By default:
-
-        %(h)s %(l)s %(u)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"
-
-
-        h: remote address
-        l: '-'
-        u: currently '-', may be user name in future releases
-        t: date of the request
-        r: status line (ex: GET / HTTP/1.1)
-        s: status
-        b: response length or '-'
-        f: referer
-        a: user agent
-        T: request time in seconds
-        D: request time in microseconds,
-        p: process ID
-        {Header}i: request header
-        {Header}o: response header
+        ==========  ===========
+        Identifier  Description
+        ==========  ===========
+        h           remote address
+        l           '-'
+        u           currently '-', may be user name in future releases
+        t           date of the request
+        r           status line (e.g. ``GET / HTTP/1.1``)
+        s           status
+        b           response length or '-'
+        f           referer
+        a           user agent
+        T           request time in seconds
+        D           request time in microseconds
+        L           request time in decimal seconds
+        p           process ID
+        {Header}i   request header
+        {Header}o   response header
+        ==========  ===========
         """
 
 
@@ -888,7 +1047,7 @@ class ErrorLog(Setting):
     cli = ["--error-logfile", "--log-file"]
     meta = "FILE"
     validator = validate_string
-    default = "-"
+    default = None
     desc = """\
         The Error log file to write to.
 
@@ -922,7 +1081,7 @@ class LoggerClass(Setting):
     cli = ["--logger-class"]
     meta = "STRING"
     validator = validate_class
-    default = "simple"
+    default = "gunicorn.glogging.Logger"
     desc = """\
         The logger you want to use to log events in gunicorn.
 
@@ -944,10 +1103,11 @@ class LogConfig(Setting):
     validator = validate_string
     default = None
     desc = """\
-The log config file to use.
-Gunicorn uses the standard Python logging module's Configuration
-file format.
-"""
+    The log config file to use.
+    Gunicorn uses the standard Python logging module's Configuration
+    file format.
+    """
+
 
 class SyslogTo(Setting):
     name = "syslog_addr"
@@ -966,7 +1126,16 @@ class SyslogTo(Setting):
         default = "udp://localhost:514"
 
     desc = """\
-    Address to send syslog messages
+    Address to send syslog messages.
+
+    Address is a string of the form:
+
+    * 'unix://PATH#TYPE' : for unix domain socket. TYPE can be 'stream'
+      for the stream driver or 'dgram' for the dgram driver.
+      'stream' is the default.
+    * 'udp://HOST:PORT' : for UDP sockets
+    * 'tcp://HOST:PORT' : for TCP sockets
+
     """
 
 
@@ -978,7 +1147,7 @@ class Syslog(Setting):
     action = 'store_true'
     default = False
     desc = """\
-    Log to syslog.
+    Send *Gunicorn* logs to syslog.
     """
 
 
@@ -1063,10 +1232,12 @@ class DjangoSettings(Setting):
     validator = validate_string
     default = None
     desc = """\
-        The Python path to a Django settings module.
+        The Python path to a Django settings module. (deprecated)
 
         e.g. 'myproject.settings.main'. If this isn't provided, the
         DJANGO_SETTINGS_MODULE environment variable will be used.
+
+        **DEPRECATED**: use the --env argument instead.
         """
 
 
@@ -1078,13 +1249,30 @@ class PythonPath(Setting):
     validator = validate_string
     default = None
     desc = """\
-        A directory to add to the Python path for Django.
+        A directory to add to the Python path.
 
         e.g.
         '/home/djangoprojects/myproject'.
         """
 
 
+class Paste(Setting):
+    name = "paste"
+    section = "Server Mechanics"
+    cli = ["--paste", "--paster"]
+    meta = "STRING"
+    validator = validate_string
+    default = None
+    desc = """\
+        Load a paste.deploy config file. The argument may contain a "#" symbol
+        followed by the name of an app section from the config file, e.g.
+        "production.ini#admin".
+
+        At this time, using alternate server blocks is not supported. Use the
+        command line arguments to control server configuration instead.
+        """
+
+
 class OnStarting(Setting):
     name = "on_starting"
     section = "Server Hooks"
@@ -1184,6 +1372,43 @@ class PostWorkerInit(Setting):
         Worker.
         """
 
+class WorkerInt(Setting):
+    name = "worker_int"
+    section = "Server Hooks"
+    validator = validate_callable(1)
+    type = six.callable
+
+    def worker_int(worker):
+        pass
+
+    default = staticmethod(worker_int)
+    desc = """\
+        Called just after a worker exited on SIGINT or SIGQUIT.
+
+        The callable needs to accept one instance variable for the initialized
+        Worker.
+        """
+
+
+class WorkerAbort(Setting):
+    name = "worker_abort"
+    section = "Server Hooks"
+    validator = validate_callable(1)
+    type = six.callable
+
+    def worker_abort(worker):
+        pass
+
+    default = staticmethod(worker_abort)
+    desc = """\
+        Called when a worker received the SIGABRT signal.
+
+        This call generally happen on timeout.
+
+        The callable needs to accept one instance variable for the initialized
+        Worker.
+        """
+
 
 class PreExec(Setting):
     name = "pre_exec"
@@ -1271,6 +1496,21 @@ class NumWorkersChanged(Setting):
         None.
         """
 
+class OnExit(Setting):
+    name = "on_exit"
+    section = "Server Hooks"
+    validator = validate_callable(1)
+
+    def on_exit(server):
+        pass
+
+    default = staticmethod(on_exit)
+    desc = """\
+        Called just before exiting gunicorn.
+
+        The callable needs to accept a single instance variable for the Arbiter.
+        """
+
 
 class ProxyProtocol(Setting):
     name = "proxy_protocol"
@@ -1289,12 +1529,12 @@ class ProxyProtocol(Setting):
 
         Example for stunnel config::
 
-        [https]
-        protocol = proxy
-        accept  = 443
-        connect = 80
-        cert = /etc/ssl/certs/stunnel.pem
-        key = /etc/ssl/certs/stunnel.key
+            [https]
+            protocol = proxy
+            accept  = 443
+            connect = 80
+            cert = /etc/ssl/certs/stunnel.pem
+            key = /etc/ssl/certs/stunnel.key
         """
 
 
@@ -1306,6 +1546,10 @@ class ProxyAllowFrom(Setting):
     default = "127.0.0.1"
     desc = """\
         Front-end's IPs from which allowed accept proxy requests (comma separate).
+
+        Set to "*" to disable checking of Front-end IPs (useful for setups
+        where you don't know in advance the IP address of Front-end, but
+        you still trust the environment)
         """
 
 
@@ -1331,3 +1575,79 @@ class CertFile(Setting):
     desc = """\
     SSL certificate file
     """
+
+class SSLVersion(Setting):
+    name = "ssl_version"
+    section = "Ssl"
+    cli = ["--ssl-version"]
+    validator = validate_pos_int
+    default = ssl.PROTOCOL_TLSv1
+    desc = """\
+    SSL version to use (see stdlib ssl module's)
+    """
+
+class CertReqs(Setting):
+    name = "cert_reqs"
+    section = "Ssl"
+    cli = ["--cert-reqs"]
+    validator = validate_pos_int
+    default = ssl.CERT_NONE
+    desc = """\
+    Whether client certificate is required (see stdlib ssl module's)
+    """
+
+class CACerts(Setting):
+    name = "ca_certs"
+    section = "Ssl"
+    cli = ["--ca-certs"]
+    meta = "FILE"
+    validator = validate_string
+    default = None
+    desc = """\
+    CA certificates file
+    """
+
+class SuppressRaggedEOFs(Setting):
+    name = "suppress_ragged_eofs"
+    section = "Ssl"
+    cli = ["--suppress-ragged-eofs"]
+    action = "store_true"
+    default = True
+    validator = validate_bool
+    desc = """\
+    Suppress ragged EOFs (see stdlib ssl module's)
+    """
+
+class DoHandshakeOnConnect(Setting):
+    name = "do_handshake_on_connect"
+    section = "Ssl"
+    cli = ["--do-handshake-on-connect"]
+    validator = validate_bool
+    action = "store_true"
+    default = False
+    desc = """\
+    Whether to perform SSL handshake on socket connect (see stdlib ssl module's)
+    """
+
+if sys.version_info >= (2, 7):
+    class Ciphers(Setting):
+        name = "ciphers"
+        section = "Ssl"
+        cli = ["--ciphers"]
+        validator = validate_string
+        default = 'TLSv1'
+        desc = """\
+        Ciphers to use (see stdlib ssl module's)
+        """
+
+# statsD monitoring
+class StatsdHost(Setting):
+    name = "statsd_host"
+    section = "Logging"
+    cli = ["--statsd-host"]
+    meta = "STATSD_ADDR"
+    default = "localhost:8125"
+    validator = validate_hostport
+    desc ="""\
+    host:port of the statsd server to log to
+    """
